' =========================================================================
'
'   File...... PMMVGD_NTSC_HDR.INC (C) Copyright 2008 Hitt Consulting
'   Compiler.. SX/B Version 1.51.03
'   Purpose... Pico Mere Mortal Video Game Driver for NTSC
'   Author.... Terry Hitt
'   E-mail.... terry@hittconsulting.com
'   Started... Jan 28, 2008
'   Updated...
'   
' =========================================================================

' Program Description
'  Allow 16 tiles 16x11 screen
'  1 Sound channel, 16 volume levels (0 to 15)
'  Last tile in each set should be blank (or background) because the scroll routines fill with this tile
'
' Constants:
'  Pressed    Used to determine if a joystick button is pressed as in "IF JStkUp = Pressed THEN"
'
'  
' Routines:
'  PutTile X, Y, TileID or Pos, TileID
'  GetTile X, Y or Pos; returns TileID and RAM location
'  Snd tone,duration
'  Delay value
'  WaitSync
'
' Optional Routines:
'  LineAddr line
'  ClearScreen TileID{,line}
'  ScrollUp
'  ScrollDown
'  ScrollLeft
'  ScrollRight
'  ValueInc position
'  ValueDec position
'  PlotXY xPos, yPos
'  UnPlotXY xPos, yPos

'
' Conditional Compilation Defines:
'  TILESETS_1        If defined then only one tile set is used for all video lines (more program space for game)
'  TILESETS_2        If defined then two tile sets are used (default) (less program space for game)
'
'  PIXEL_TILESET_1   If defined sets tileset 1 to patterns for pixels (see PlotXY & UnPlotXY)
'  PIXEL_TILESET_2   If defined sets tileset 2 to patterns for pixels (see PlotXY & UnPlotXY)

'  LOAD_WAITSYNC    
'  LOAD_LINEADDR
'  LOAD_CLEARSCREEN
'  LOAD_SCROLLUP
'  LOAD_SCROLLDOWN
'  LOAD_SCROLLLEFT
'  LOAD_SCROLLRIGHT
'  LOAD_VALUEINC
'  LOAD_VALUEDEC
'  LOAD_PLOTXY
'  LOAD_UNPLOTXY
'
' Subroutine Explaination
'   Delay value             Delays for "value" milliseconds. "value" may be a byte, word or constant
'   WaitSync                Waits for the next video sync
'   LineAddr line           Returns the RAM address of the start of video line "line"  
'   ClearScreen tile{,line} Fills video memory starting at line "line" with tile "tile"
'   SCROLLUP                Scrolls lines up, does not affect line 0, clears bottom line
'   SCROLLDOWN              Scrolls lines down, does not affect line 0, clears line 1
'   SCROLLLEFT              Scrolls left, does not affect line 0
'   SCROLLRIGHT             Scrolls right, does not affect line 0
'   VALUEINC position       Increments value where "position" is the last (1's) digit
'   VALUEDEC position       Decrements value where "position" is the last (1's) digit
'   PlotXY xPos,yPos        Plots a point at xPos, yPos. TileSet must be the pixel tileset
'   UnPlotXY xPos,yPos      UnPlots a point at xPos, yPos. TileSet must be the pixel tileset


' -------------------------------------------------------------------------
' RC.0 to RC.3 = Video DAC
' RC.4 to RC.7 = Audio DAC
'
'
' Memeory map:
'  For a single tileset (with TILE_SETS_1 defined}
'    $000 - $1FF = MMVGM Driver Code
'    $200 - $27F = Tile Set 1 (Bitmaps for tile set)
'    $280 - $7FE = Game program
'
'  For two tilesets (top line uses it's own tileset)
'    $000 - $1FF = MMVGM Driver Code
'    $200 - $27F = Tile Set 1 (Line 0 bitmaps for tile set)
'    $280 - $2fF = Tile Set 2 (Lines 1-10 bitmaps for tile set)
'    $300 - $7FE = Game program


' -------------------------------------------------------------------------
' Device Settings
' -------------------------------------------------------------------------

DEVICE SX28, TURBO, STACKX, OPTIONX, OSCHS1, BOR26
FREQ 80_000_000  ' Use the 80 MHz Pico Oscillator

' -------------------------------------------------------------------------
' IO Pins
' -------------------------------------------------------------------------
LEDs            PIN RA OUTPUT

JStkUp          PIN RB.0 INPUT PULLUP  ' Joystick input pins
JStkDown        PIN RB.1 INPUT PULLUP  ' Joystick input pins
JStkLeft        PIN RB.2 INPUT PULLUP  ' Joystick input pins
JStkRight       PIN RB.3 INPUT PULLUP  ' Joystick input pins
JStkFire        PIN RB.4 INPUT PULLUP  ' Joystick input pins
IO_0            PIN RB.5 INPUT PULLUP
IO_1            PIN RB.6 INPUT PULLUP
IO_2            PIN RB.7 INPUT PULLUP

AVPort          PIN RC OUTPUT          ' Connects to Audio/Video R2R DACs
AVBlack         PIN RC.2 OUTPUT        ' Make high for black output
AVWhite         PIN RC.3 OUTPUT        ' Make high (along with black) for white output



' -------------------------------------------------------------------------
' Constants
' -------------------------------------------------------------------------
Pressed         CON 0                  ' Buttons go to zero when pressed

Top_Border      CON 40 + 8             ' Video lines in top border
Bot_Border      CON 24 + 8             ' Video lines in bottom border
Line_Rept       CON 2                  ' # of times to repeat each video line 2=NTSC
Left_Bdr        CON 160                ' Width of left boarder (moves display left & right)

VidMode_VSync   CON 0                  ' Video modes: 0 = Generating Vertical Sync
VidMode_TBlank  CON 1                  '              1 = Generating Top Blank Lines
VidMode_Active  CON 2                  '              2 = Generating Active Video Lines
VidMode_BBlank  CON 3                  '              3 = Generating Bottom Blank Lines

Line0           CON 0                  ' Offset for video line 0
Line1           CON 16                 ' Offset for video line 1
Line2           CON 32                 ' Offset for video line 2
Line3           CON 48                 ' Offset for video line 3
Line4           CON 64                 ' Offset for video line 4
Line5           CON 80                 ' Offset for video line 5
Line6           CON 96                 ' Offset for video line 6
Line7           CON 112                ' Offset for video line 7
Line8           CON 128                ' Offset for video line 8
Line9           CON 144                ' Offset for video line 9
Line10          CON 160                ' Offset for video line 10

' -------------------------------------------------------------------------
' Variables
' -------------------------------------------------------------------------
' Arrays for text storage
videoLine0      VAR BYTE(8)            ' 1st line of text (line 0)
videoLine1      VAR BYTE(8)            ' 2st line of text (line 1)
videoLine2      VAR BYTE(8)            ' 3rd line of text (line 2)
videoLine3      VAR BYTE(8)            ' 4th line of text (line 3)
videoLine4      VAR BYTE(8)            ' 5th line of text (line 4)
videoLine5      VAR BYTE(8)            ' 6th line of text (line 5)
videoLine6      VAR BYTE(8)            ' 7th line of text (line 6)
videoLine7      VAR BYTE(8)            ' 8th line of text (line 7)
videoLine8      VAR BYTE(8)            ' 9th line of text (line 8)
videoLine9      VAR BYTE(8)            ' 10th line of text (line 9)
videoLine10     VAR BYTE(8)            ' 11th line of text (line 10)


' Video routine variables
vidCnt          VAR BYTE (1)           ' vsync=6, topblank=40, active=240, botblank=24
vidMode         VAR BYTE (1)           ' 0=vsync, 1=topblank, 2=active, 3=botblank
vidLineRpt      VAR BYTE (1)           ' Repeat each line counter
vidTileLine     VAR BYTE               ' Current line within 1 tile (cannot be an array)
tileAddr        VAR BYTE               ' RAM address of tile (cannot be an array)
tileDots        VAR BYTE               ' Tile line dot pattern (cannot be an array)
sndDurCnt       VAR BYTE (1)           ' Counts frames for sound duration
sndToneCnt      VAR BYTE (1)           ' Counts video lines for sound
sndTone         VAR BYTE (1)           ' Current sound tone value
sndVol          VAR BYTE (1)           ' Volume of current sound
intFlag         VAR BIT                ' Cleared by video interrupt

' -------------------------------------------------------------------------
' Interrupt Code
' -------------------------------------------------------------------------
INTERRUPT NOPRESERVE
  GOTO IntHandler


' -------------------------------------------------------------------------
' Set program starting label
' -------------------------------------------------------------------------
PROGRAM Initialize NOSTARTUP


' -------------------------------------------------------------------------
' Subroutines
' -------------------------------------------------------------------------
' Subroutines used by interrupt
PrimeTileDots SUB 0                    ' Used by video output routine
VideoPause    SUB 0                    ' Used by video output routine

' Subroutines for use in main program
PutTile       SUB 2, 3                 ' X, Y, TileID or Pos, TileID
GetTile       FUNC 2,1,2               ' X, Y or Pos; Returns TileID and RAM location
Snd           SUB 3                    ' Freq,Duration,Volume
Delay         SUB                      ' Delays for about "x" milliseconds
WaitSync      SUB 0                    ' Waits for the active video field to be done


' Load optional routines if they are defined. 
' NOTE these take up user program space, so don't load them unless you need them.

'{$IFDEF LOAD_LINEADDR}
LineAddr      FUNC 1,1                 ' Given line# returns address
'{$ENDIF}

'{$IFDEF LOAD_CLEARSCREEN}
ClearScreen   SUB 1, 2                 ' TileID or TileID, Start Line
'{$ENDIF}

'{$IFDEF LOAD_SCROLLUP}
ScrollUp      SUB 0,1                  ' optional tile to clear new line
'{$ENDIF}

'{$IFDEF LOAD_SCROLLDOWN}
ScrollDown    SUB 0,1                  ' optional tile to clear new line
'{$ENDIF}

'{$IFDEF LOAD_SCROLLLEFT}
ScrollLeft    SUB 1                    ' tile to clear new line
'{$ENDIF}

'{$IFDEF LOAD_SCROLLRIGHT}
ScrollRight   SUB 1                    ' tile to clear new line
'{$ENDIF}

'{$IFDEF LOAD_VALUEINC}
ValueInc      SUB 1                    ' Position
'{$ENDIF}

'{$IFDEF LOAD_VALUEDEC}
ValueDec      SUB 1                    ' Position
'{$ENDIF}

'{$IFDEF LOAD_PLOTXY}
PlotXY            SUB 2                ' x,y (x=0 to 31, y=0 to 21)
'{$ENDIF}

'{$IFDEF LOAD_UNPLOTXY}
UnPlotXY          SUB 2                ' x,y (x=0 to 31, y=0 to 21)
'{$ENDIF}

